/**
 * JOnAS Configurator
 * Copyright (C) 2008-2010 Bull S.A.S.
 * Copyright (C) 2008-2010 France Telecom R&D
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Jonas4.java 22223 2012-03-08 19:28:08Z cazauxj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.tools.configurator.impl;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.listener.TimestampedLogger;
import org.objectweb.jonas.ant.JOnASBaseTask;
import org.objectweb.jonas.ant.JOnASClusterConfigTask;
import org.objectweb.jonas.ant.cluster.ClusterDaemon;
import org.objectweb.jonas.ant.jonasbase.Carol;
import org.objectweb.jonas.ant.jonasbase.Db;
import org.objectweb.jonas.ant.jonasbase.Discovery;
import org.objectweb.jonas.ant.jonasbase.Ha;
import org.objectweb.jonas.ant.jonasbase.JdbcRa;
import org.objectweb.jonas.ant.jonasbase.Jms;
import org.objectweb.jonas.ant.jonasbase.JonasProperties;
import org.objectweb.jonas.ant.jonasbase.Mail;
import org.objectweb.jonas.ant.jonasbase.Services;
import org.objectweb.jonas.ant.jonasbase.WebContainer;
import org.objectweb.jonas.ant.jonasbase.web.Ajp;
import org.objectweb.jonas.ant.jonasbase.web.Cluster;
import org.objectweb.jonas.ant.jonasbase.web.Http;
import org.objectweb.jonas.ant.jonasbase.web.Https;
import org.objectweb.jonas.ant.jonasbase.web.Tomcat;
import org.objectweb.jonas.ant.jonasbase.wsdl.WsdlPublish;
import org.ow2.jonas.tools.configurator.api.JDBCConfiguration;
import org.ow2.jonas.tools.configurator.api.JdbcXMLConfiguration;
import org.ow2.jonas.tools.configurator.api.JonasConfigurator;

/**
 * JonasConfigurator implementation for Jonas 4.x
 * 
 * @author Remy Bresson
 */
public class Jonas4 extends FileReplacerHelper implements JonasConfigurator {

    /**
     * JONAS_BASE
     */
    private File jonasFileBase;

    /**
     * Link to the jonas ant task
     */
    private JOnASBaseTask jonasBase;

    /**
     * The cluster daemon configuration task
     */
    private JOnASClusterConfigTask clusterDaemonTask;

    /**
     * The cluster daemon confguration
     */
    private ClusterDaemon clusterDaemon;

    /**
     * Configuration logger
     */
    private static Logger logger = Logger.getLogger(Jonas4.class.getName());

    /**
     * To configure protocols
     */
    private Carol carol;

    /**
     * Has JOnAS protocols list been set?
     */
    private boolean isProtocolsListSet = false;

    /**
     * To configure databases service
     */
    private Db db;

    /**
     * Whether the optional db service has been activated
     */
    private boolean isDbActivated = false;

    /**
     * To configure discovery sevice
     */
    private Discovery discovery;

    /**
     * Whether Discovery is set as master
     */
    private boolean isDiscoveryMaster = false;

    /**
     * To configure jdbc
     */
    private Map<String, JdbcRa> jdbcRa;

    /**
     * To configure jms
     */
    private Jms jms;

    /**
     * To configure jonas global properties
     */
    private JonasProperties jonasProperties;

    /**
     * To configure mail service
     */
    private Mail mail;

    /**
     * Whether the optional mail service has been activated
     */
    private boolean isMailActivated = false;

    /**
     * To configure active services
     */
    private Services services;

    /**
     * To configure web container
     */
    private WebContainer webContainer;

    /**
     * To configure the tomcat (in the web container)
     */
    private Tomcat tomcat;

    /**
     * To configure ajp
     */
    private Ajp ajp;

    /**
     * To configure http
     */
    private Http http;

    /**
     * To configure https
     */
    private Https https;

    /**
     * To configure cluster
     */
    private Cluster cluster;

    /**
     * To configure wsdl wsdlPublisherFile
     */
    private org.objectweb.jonas.ant.jonasbase.wsdl.File wsdlPublisherFile;

    /**
     * To configure WSDL publishing
     */
    private WsdlPublish wsdlPublish;

    /**
     * Whether the optional WSDL publishing service has been activated
     */
    private boolean isWsdlPublishActivated = false;

    /**
     * To configure ha
     */
    private Ha ha;

    /**
     * The project, not really usefull, only to have a project to pass to ant
     * task.
     */
    private Project project;

    /**
     * Save the jonas root to be able to re use it after setting
     */
    private boolean isJonasRootSet = false;

    /**
     * Save the jonas base to be able to re use it after setting
     */
    private boolean isJonasBaseSet = false;

    /**
     * Is http webservice activated
     */
    private Boolean isHttpActivated = true;

    /**
     * Is https webservice activated
     */
    private Boolean isHttpsActivated = false;

    /**
     * Is ajp webservice activated
     */
    private Boolean isAjpActivated = false;

    /**
     * Is http replication webservice activated
     */
    private Boolean isHttpReplicationActivated = false;

    /**
     * Is ha activated
     */
    private Boolean isHaActivated = false;

    /**
     * Is ejb clustering activated
     */
    private Boolean isEjbClusteringActivated = false;

    /**
     * Save the value of mcast_addr for cmi configuration
     */
    private String saveCmiMCastAddr;

    /**
     * Save the value of mcast_port for cmi configuration
     */
    private String saveCmiMCastPort;

    /**
     * Default constructor
     */
    public Jonas4() {
        this.jonasBase = new JOnASBaseTask();
        this.createProject();
        this.jonasBase.setProject(this.project);

        this.clusterDaemonTask = new JOnASClusterConfigTask();
        this.clusterDaemonTask.setProject(this.project);

        this.carol = new Carol();
        this.db = new Db();
        this.discovery = new Discovery();
        this.jdbcRa = new HashMap<String, JdbcRa>();

        this.jms = new Jms();
        try {
            this.jonasProperties = new JonasProperties();
        } catch (NoClassDefFoundError e) {
            // JOnAS before revision 12011, which is just after the 4.8.6
            // release, didn't have JonasProperties. We'll handle that manually.
            this.jonasProperties = null;
            this.services = new Services();
        }
        this.mail = new Mail();

        this.webContainer = new WebContainer();
        this.tomcat = new Tomcat();
        this.ajp = new Ajp();
        this.http = new Http();
        this.https = new Https();
        this.cluster = new Cluster();
        this.wsdlPublish = new WsdlPublish();
        this.wsdlPublisherFile = new org.objectweb.jonas.ant.jonasbase.wsdl.File();
        this.ha = new Ha();

        this.clusterDaemon = new ClusterDaemon();
        this.clusterDaemon.setProtocol("jrmp");
        this.clusterDaemon.setPort("1099");

        Jonas4.logger.finest("Created the JOnAS 4 configurator instance");
    }

    /**
     * Create the project
     */
    private void createProject() {
        this.project = new Project();
        this.project.init();
        this.project.initProperties();

        TimestampedLogger logger = new TimestampedLogger();
        logger.setMessageOutputLevel(Project.MSG_DEBUG);

        logger.setOutputPrintStream(new PrintStream(new OutputStream() {
            StringBuffer buf = new StringBuffer();

            @Override
            public void write(final int b) throws IOException {
                if (b == '\n' || b == '\r') {
                    if (this.buf.length() > 0) {
                        Jonas4.logger.finest(this.buf.toString());
                        this.buf.setLength(0);
                    }
                } else {
                    this.buf.append((char) b);
                }
            }
        }));

        this.project.addBuildListener(logger);
    }

    /* ------------------------------- */
    /* Jonas global parameters */

    /**
     * {@inheritDoc}
     */
    public void setJdk(final String jdk) {
        Jonas4.logger.finest("[GlobalParameters] setting jdk : " + jdk);
        NotApplicableHelper.notApplicable("GlobalParameters.Jdk");
    }

    /**
     * {@inheritDoc}
     */
    public void setJavaOpts(final String javaOpts) {
        Jonas4.logger.finest("[GlobalParameters] setting java opts : " + javaOpts);
        if (this.jonasProperties != null) {
            this.jonasProperties.setJvmopts(javaOpts);
        } else {
            // Setting seems not to be applied in any wsdlPublisherFile,
            // skipping
        }
    }

    /**
     * {@inheritDoc}
     */
    public void setJonasRoot(final String jonasRoot) {
        Jonas4.logger.finest("[GlobalParameters] setting jonas root : " + jonasRoot);
        File jonasFileRoot = new File(jonasRoot);
        this.jonasBase.setJonasRoot(jonasFileRoot);
        this.discovery.setJonasRoot(jonasRoot);
        this.isJonasRootSet = true;
    }

    /**
     * {@inheritDoc}
     */
    public void setJonasBase(final String jonasBase) {
        Jonas4.logger.finest("[GlobalParameters] setting jonas base : " + jonasBase);
        this.jonasFileBase = new File(jonasBase);
        this.jonasBase.setDestDir(this.jonasFileBase);
        this.isJonasBaseSet = true;
    }

    /**
     * {@inheritDoc}
     */
    public void isJOnASBaseToUpdate(final Boolean update) {
        Jonas4.logger.finest("[GlobalParameters] setting isJOnASBaseToUpdate : " + update);
        NotApplicableHelper.notApplicable("isJOnASBaseToUpdate");
    }

    /**
     * {@inheritDoc}
     */
    public void setJonasName(final String jonasName) {
        Jonas4.logger.finest("[GlobalParameters] setting jonas name : " + jonasName);
        if (this.jonasProperties != null) {
            this.jonasProperties.setServerName(jonasName);
        } else {
            // Setting seems not to be applied in any wsdlPublisherFile,
            // skipping
        }
    }

    /**
     * {@inheritDoc}
     */
    public void setJonasDomain(final String jonasDomain) {
        Jonas4.logger.finest("[GlobalParameters] setting jonas domain : " + jonasDomain);
        if (this.jonasProperties != null) {
            this.jonasProperties.setDomainName(jonasDomain);
        } else {
            // Setting seems not to be applied in any wsdlPublisherFile,
            // skipping
        }
    }

    /**
     * {@inheritDoc}
     */
    public void setHost(final String jonasHost) {
        Jonas4.logger.finest("[GlobalParameters] setting host : " + jonasHost);
        this.addReplacement("carol.properties", "://localhost:", "://" + jonasHost + ":");
        NotApplicableHelper.notApplicable("GlobalParameters.JMSHost");
    }

    /**
     * {@inheritDoc}
     */
    public void setJonasDevelopment(final boolean development) {
        Jonas4.logger.finest("[GlobalParameters] setting development : " + development);
        NotApplicableHelper.notApplicable("GlobalParameters.development");
    }

    /**
     * {@inheritDoc}
     */
    public void setLogConfigFile(final String logConfigFile) {
        Jonas4.logger.finest("[Bootstrap] setting log config file : " + logConfigFile);
        NotApplicableHelper.notApplicable("Bootstrap.logConfigFile");
    }

    /**
     * {@inheritDoc}
     */
    public void setMaster(final Boolean isMaster) {
        Jonas4.logger.finest("[Bootstrap] setting master : " + isMaster);
        NotApplicableHelper.notApplicable("Bootstrap.isMaster");
    }

    /**
     * {@inheritDoc}
     */
    public void setSecurityPropagation(final Boolean securityPropagation) {
        Jonas4.logger.finest("[Bootstrap] setting security propagation : " + securityPropagation);
        NotApplicableHelper.notApplicable("Bootstrap.securityPropagation");
    }

    /**
     * {@inheritDoc}
     */
    public void setTransactionPropagation(final Boolean transactionPropagation) {
        Jonas4.logger.finest("[Bootstrap] setting transaction propagation : " + transactionPropagation);
        NotApplicableHelper.notApplicable("Bootstrap.transactionPropagation");
    }

    /**
     * {@inheritDoc}
     */
    public void setCsiv2Propagation(final Boolean csiv2Propagation) {
        Jonas4.logger.finest("[Bootstrap] setting csiv2 propagation : " + csiv2Propagation);
        NotApplicableHelper.notApplicable("Bootstrap.csiv2Propagation");
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Jonas protocols configuation */
    /**
     * {@inheritDoc}
     */
    public void setProtocolsList(final String protocolsList) {
        Jonas4.logger.finest("[Protocols configuration] setting protocols list : " + protocolsList);
        this.carol.setProtocols(protocolsList);
        this.isProtocolsListSet = true;
    }

    /**
     * {@inheritDoc}
     */
    public void setProtocolsIrmiPort(final String irmiPort) {
        Jonas4.logger.finest("[Protocols configuration] setting irmi port : " + irmiPort);
        this.carol.setIrmiPort(irmiPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setProtocolsIiopPort(final String iiopPort) {
        Jonas4.logger.finest("[Protocols configuration] setting iiop port : " + iiopPort);
        this.carol.setIiopPort(iiopPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setProtocolsJrmpPort(final String jrmpPort) {
        Jonas4.logger.finest("[Protocols configuration] setting jrmp port : " + jrmpPort);
        this.carol.setJrmpPort(jrmpPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setProtocolsJrmLocalRegistry(final Boolean localRegistry) {
        Jonas4.logger.finest("[Protocols configuration] setting jrmp local registry : " + localRegistry);
        NotApplicableHelper.notApplicable("Carol.jrmpLocalRegistry");
    }

    /**
     * {@inheritDoc}
     */
    public void setCarolHost(final String host) {
        Jonas4.logger.finest("[Protocols configuration] setting host : " + host);
        NotApplicableHelper.notApplicable("Carol.host");
    }

    /**
     * {@inheritDoc}
     */
    public void setProtocolsCmiPort(final String cmiPort) {
        Jonas4.logger.finest("[Protocols configuration] setting cmi port : " + cmiPort);
        this.carol.setCmiPort(cmiPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setProtocolsLocalCallOptimized(final Boolean localCallOptimized) {
        Jonas4.logger.finest("[Protocols configuration] setting jndi local call optimisation : " + localCallOptimized);
        this.carol.setJrmpOptimization(localCallOptimized);
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Active services */

    /**
     * {@inheritDoc}
     */
    public void setServices(final String services) {
        Jonas4.logger.finest("setting active services list : " + services);
        if (this.jonasProperties != null) {
            this.jonasProperties.setServices(services);
        } else {
            this.services.setNames(services);
        }
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Service web configuration */
    /**
     * {@inheritDoc}
     */
    public void setWebcontainer(final String webContainer) {
        Jonas4.logger.finest("[Web configuration] setting webContainer : " + webContainer);
        NotApplicableHelper.notApplicable("Web.webContainer");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpConnectorActivation(final Boolean activation) {
        Jonas4.logger.finest("[Web configuration] setting isHttpActivated : " + activation);
        NotApplicableHelper.notApplicable("Web.isHttpActivated");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsConnectorActivation(final Boolean activation) {
        Jonas4.logger.finest("[Web configuration] setting isHttpsActivated : " + activation);
        this.isHttpsActivated = activation;
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpConnectorActivation(final Boolean activation) {
        Jonas4.logger.finest("[Web configuration] setting isAjpActivated : " + activation);
        this.isAjpActivated = activation;
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpSessionReplicationActivation(final Boolean activation) {
        Jonas4.logger.finest("[Web configuration] setting isHttpReplicationActivated : " + activation);
        this.isHttpReplicationActivated = activation;
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpSessionManagerActivation(final Boolean activation) {
        Jonas4.logger.finest("[Web configuration] setting isHttpSessionManagerActivated:" + activation);
        NotApplicableHelper.notApplicable("HTTP.isHttpSessionManagerActivated");
    }

    /**
     * {@inheritDoc}
     */
    public void setWebOndemandActivation(final Boolean activation) {
        Jonas4.logger.finest("[Web configuration] setting isWebOndemandActivated:" + activation);
        NotApplicableHelper.notApplicable("Web.ondemand");
    }

    /**
     * {@inheritDoc}
     */
    public void setWebOndemandRedirectPort(final String redirectPort) {
        Jonas4.logger.finest("[Web configuration] setting web ondemand redirect port:" + redirectPort);
        NotApplicableHelper.notApplicable("Web.ondemand.redirectPort");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpPort(final String httpPort) {
        Jonas4.logger.finest("[Web configuration] setting http port : " + httpPort);
        this.http.setPort(httpPort);
        this.addReplacement("uddi.properties", ":9000/", ":" + httpPort + "/");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpMaxThreads(final String httpMaxThreads) {
        Jonas4.logger.finest("[Web configuration] setting max http threads : " + httpMaxThreads);
        NotApplicableHelper.notApplicable("HTTP.maxThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpMinSpareThreads(final String httpMinSpareThreads) {
        Jonas4.logger.finest("[Web configuration] setting min http spare threads : " + httpMinSpareThreads);
        NotApplicableHelper.notApplicable("HTTP.minSpareThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpMaxSpareThreads(final String httpMaxSpareThreads) {
        Jonas4.logger.finest("[Web configuration] setting max http spare threads : " + httpMaxSpareThreads);
        NotApplicableHelper.notApplicable("HTTP.maxSpareThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpEnableLookups(final Boolean enableLookups) {
        Jonas4.logger.finest("[Web configuration] setting http enable lookups:" + enableLookups);
        NotApplicableHelper.notApplicable("HTTP.enableLookups");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpConnectionTimeout(final String connectionTimeout) {
        Jonas4.logger.finest("[Web configuration] setting http connection timeout:" + connectionTimeout);
        NotApplicableHelper.notApplicable("HTTP.connectionTimeout");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpAcceptCount(final String acceptCount) {
        Jonas4.logger.finest("[Web configuration] setting http accept count:" + acceptCount);
        NotApplicableHelper.notApplicable("HTTP.acceptCount");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpMaxKeepAliveRequest(final String maxKeepAliveRequest) {
        Jonas4.logger.finest("[Web configuration] setting max keep alive http request:" + maxKeepAliveRequest);
        NotApplicableHelper.notApplicable("HTTP.maxKeepAliveRequest");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpCompression(final String compression) {
        Jonas4.logger.finest("[Web configuration] setting http compression:" + compression);
        NotApplicableHelper.notApplicable("HTTP.compression");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpRedirectPort(final String redirectPort) {
        Jonas4.logger.finest("[Web configuration] setting http redirect port:" + redirectPort);
        NotApplicableHelper.notApplicable("HTTP.redirectPort");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsPort(final String httpsPort) {
        Jonas4.logger.finest("[Web configuration] setting https port : " + httpsPort);
        this.https.setPort(httpsPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsMaxThreads(final String httpsMaxThreads) {
        Jonas4.logger.finest("[Web configuration] setting max https threads : " + httpsMaxThreads);
        NotApplicableHelper.notApplicable("HTTPS.maxThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsMinSpareThreads(final String httpsMinSpareThreads) {
        Jonas4.logger.finest("[Web configuration] setting min https spare threads : " + httpsMinSpareThreads);
        NotApplicableHelper.notApplicable("HTTPS.minSpareThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsMaxSpareThreads(final String httpsMaxSpareThreads) {
        Jonas4.logger.finest("[Web configuration] setting max https spare threads : " + httpsMaxSpareThreads);
        NotApplicableHelper.notApplicable("HTTPS.maxSpareThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsEnableLookups(final Boolean enableLookups) {
        Jonas4.logger.finest("[Web configuration] setting https enable lookups:" + enableLookups);
        NotApplicableHelper.notApplicable("HTTPS.enableLookups");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsConnectionTimeout(final String connectionTimeout) {
        Jonas4.logger.finest("[Web configuration] setting https connection timeout:" + connectionTimeout);
        NotApplicableHelper.notApplicable("HTTPS.connectionTimeout");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsAcceptCount(final String acceptCount) {
        Jonas4.logger.finest("[Web configuration] setting https accept count:" + acceptCount);
        NotApplicableHelper.notApplicable("HTTPS.acceptCount");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsMaxKeepAliveRequest(final String maxKeepAliveRequest) {
        Jonas4.logger.finest("[Web configuration] setting max keep alive https request:" + maxKeepAliveRequest);
        NotApplicableHelper.notApplicable("HTTPS.maxKeepAliveRequest");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsCompression(final String compression) {
        Jonas4.logger.finest("[Web configuration] setting https compression:" + compression);
        NotApplicableHelper.notApplicable("HTTPS.compression");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsRedirectPort(final String redirectPort) {
        Jonas4.logger.finest("[Web configuration] setting https redirect port:" + redirectPort);
        NotApplicableHelper.notApplicable("HTTPS.redirectPort");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsKeystoreFile(final String keystoreFile) {
        Jonas4.logger.finest("[Web configuration] setting https keystore file:" + keystoreFile);
        NotApplicableHelper.notApplicable("HTTPS.keystoreFile");
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpsKeystorePass(final String keystorePass) {
        Jonas4.logger.finest("[Web configuration] setting https keystore password:" + keystorePass);
        NotApplicableHelper.notApplicable("HTTPS.keystorePass");
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpPort(final String ajpPort) {
        Jonas4.logger.finest("[Web configuration] setting ajp port : " + ajpPort);
        this.ajp.setPort(ajpPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpMaxThreads(final String ajpMaxThreads) {
        Jonas4.logger.finest("[Web configuration] setting max ajp threads : " + ajpMaxThreads);
        NotApplicableHelper.notApplicable("AJP.maxThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpMinSpareThreads(final String ajpMinSpareThreads) {
        Jonas4.logger.finest("[Web configuration] setting min ajp spare threads : " + ajpMinSpareThreads);
        NotApplicableHelper.notApplicable("AJP.minSpareThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpMaxSpareThreads(final String ajpMaxSpareThreads) {
        Jonas4.logger.finest("[Web configuration] setting max ajp spare threads : " + ajpMaxSpareThreads);
        NotApplicableHelper.notApplicable("AJP.maxSpareThreads");
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpEnableLookups(final Boolean enableLookups) {
        Jonas4.logger.finest("[Web configuration] setting ajp enable lookups:" + enableLookups);
        NotApplicableHelper.notApplicable("AJP.enableLookups");
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpConnectionTimeout(final String connectionTimeout) {
        Jonas4.logger.finest("[Web configuration] setting ajp connection timeout:" + connectionTimeout);
        NotApplicableHelper.notApplicable("AJP.connectionTimeout");
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpAcceptCount(final String acceptCount) {
        Jonas4.logger.finest("[Web configuration] setting ajp accept count:" + acceptCount);
        NotApplicableHelper.notApplicable("AJP.acceptCount");
    }

    /**
     * {@inheritDoc}
     */
    public void setAjpRedirectPort(final String redirectPort) {
        Jonas4.logger.finest("[Web configuration] setting ajp redirect port:" + redirectPort);
        NotApplicableHelper.notApplicable("AJP.redirectPort");
    }

    /**
     * {@inheritDoc}
     */
    public void setMaxActiveWebSessions(final String maxActiveSession) {
        Jonas4.logger.finest("[Web configuration] setting max active sessions:" + maxActiveSession);
        NotApplicableHelper.notApplicable("AJP.maxActiveSession");
    }

    /**
     * {@inheritDoc}
     */
    public void setJmxSecured(final boolean enabled) {
        Jonas4.logger.finest("[Security] setting JMX security:" + enabled);
        NotApplicableHelper.notApplicable("Security.jmxSecured");
    }

    /**
     * {@inheritDoc}
     */
    public void addUser(final String username, final String password, final String roles, final String groups,
                        final String description) {
        Jonas4.logger.finest("[Security] adding a user(suername; password; roles): (" + username + ";" + password + ";"
                + roles + ")");
        NotApplicableHelper.notApplicable("Security.addUser");
    }

    /**
     * {@inheritDoc}
     */
    public void addAdmin(final String username, final String password) {
        Jonas4.logger.finest("[Security] adding an admin(suername; password): (" + username + ";" + password + ")");
        NotApplicableHelper.notApplicable("Security.addUser");
    }

    /**
     * {@inheritDoc}
     */
    public void setJvmRoute(final String jvmRoute) {
        Jonas4.logger.finest("[Web configuration] setting jvm route : " + jvmRoute);
        try {
            this.tomcat.setJvmRoute(jvmRoute);
        } catch (NoSuchMethodError e) {
            this.addReplacement("server.xml", "<Engine name=\"jonas\" defaultHost=\"localhost\">",
                "<Engine name=\"jonas\" defaultHost=\"localhost\" jvmRoute=\"" + jvmRoute + "\">");
        }
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpReplicationClusterName(final String clusterName) {
        Jonas4.logger.finest("[Web configuration] setting cluster name : " + clusterName);
        this.cluster.setName(clusterName);
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpReplicationMulticastAddress(final String mCastAddr) {
        Jonas4.logger.finest("[Web configuration] setting multi cast address : " + mCastAddr);
        this.cluster.setMcastAddr(mCastAddr);

    }

    /**
     * {@inheritDoc}
     */
    public void setHttpReplicationMulticastPort(final String mCastPort) {
        Jonas4.logger.finest("[Web configuration] setting multi cast port : " + mCastPort);
        this.cluster.setMcastPort(mCastPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpReplicationListenPort(final String listenPort) {
        Jonas4.logger.finest("[Web configuration] setting listen port : " + listenPort);
        this.cluster.setListenPort(listenPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setHttpReplicationAlgorithm(final String algorithm) {
        Jonas4.logger.finest("[Web configuration] setting http replication algorithm : " + algorithm);
        NotApplicableHelper.notApplicable("HttpReplication.algorithm");
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Service db configuration */
    /**
     * {@inheritDoc}
     */
    public void setDbPort(final String port) {
        Jonas4.logger.finest("[Database configuration] setting db port: " + port);
        this.isDbActivated = true;
        this.db.setPort(port);
        this.addReplacement("HSQL1.properties", ":9001/", ":" + port + "/");
    }

    /**
     * {@inheritDoc}
     */
    public void setDbName(final String name) {
        Jonas4.logger.finest("[Database configuration] setting db name: " + name);
        this.isDbActivated = true;
        this.addReplacement("jonas.properties", "jonas.service.db.dbname    db_jonas", "jonas.service.db.dbname    " + name);
        this.addReplacement("HSQL1.properties", "/db_jonas", "/" + name);
    }

    /**
     * {@inheritDoc}
     */
    public void setDbUsers(final String users) {
        Jonas4.logger.finest("[Database configuration] setting db users: " + users);
        this.isDbActivated = true;
        this.addReplacement("jonas.properties", "jonas.service.db.users    jonas:jonas", "jonas.service.db.users    " + users);

        String[] multipleUsers = users.split(",");
        String firstUser = multipleUsers[0];
        String[] userPassword = firstUser.split(":");
        if (userPassword.length > 1) {
            String firstUserName = userPassword[0];
            String firstPassword = userPassword[1];

            this
                .addReplacement("HSQL1.properties", "datasource.username     jonas", "datasource.username     " + firstUserName);
            this
                .addReplacement("HSQL1.properties", "datasource.password     jonas", "datasource.password     " + firstPassword);
        }
    }

    /**
     * {@inheritDoc} 
     */
    public void setDbId(final String dbId) {
        Jonas4.logger.finest("[Database configuration] setting db id: " + dbId);
        NotApplicableHelper.notApplicable("db.id");
    }

    /**
     * {@inheritDoc}
     */    
    public void addJdbcResource(final JdbcXMLConfiguration jdbcXMLConfiguration) {
        Jonas4.logger.finest("Setting JDBC XML configuration");
        NotApplicableHelper.notApplicable("addJdbcXML");
    }

    /**
     * {@inheritDoc}
     */    
    public void addJdbcResource(String ref, JdbcXMLConfiguration jdbcXMLConfiguration) {
        Jonas4.logger.finest("Setting JDBC XML configuration");
        NotApplicableHelper.notApplicable("addJdbcXML");
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Service discovery configuration */
    /**
     * {@inheritDoc}
     */
    public void setDiscoveryMasterActivated(final Boolean masterActivated) {
        Jonas4.logger.finest("[Discovery configuration] setting masterActivated : " + masterActivated);
        this.isDiscoveryMaster = masterActivated;
    }

    /**
     * {@inheritDoc}
     */
    public void setDiscoverySourcePort(final String sourcePort) {
        Jonas4.logger.finest("[Discovery configuration] setting source port : " + sourcePort);
        this.discovery.setSourcePort(sourcePort);
    }

    /**
     * {@inheritDoc}
     */
    public void setDiscoveryDomainName(final String domainName) {
        Jonas4.logger.finest("[Discovery configuration] setting domain name : " + domainName);

        if (!this.isJonasRootSet) {
            Jonas4.logger.severe("[Discovery configuration] As jonas root isn't set, domain name skipped : ");
        } else {
            this.discovery.setDomainName(domainName);
        }
    }

    /**
     * {@inheritDoc}
     */
    public void setDiscoveryGreetingPort(final String greetingPort) {
        Jonas4.logger.finest("[Discovery configuration] setting greeting port : " + greetingPort);
        this.discovery.setGreetingPort(greetingPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setDiscoveryMulticastAddress(final String multicastAddress) {
        Jonas4.logger.finest("[Discovery configuration] setting multicast address : " + multicastAddress);
        this.discovery.setMcastAddr(multicastAddress);
    }

    /**
     * {@inheritDoc}
     */
    public void setDiscoveryMulticastPort(final String multicastPort) {
        Jonas4.logger.finest("[Discovery configuration] setting multicast port : " + multicastPort);
        this.discovery.setMcastPort(multicastPort);
    }

    /**
     * {@inheritDoc}
     */
    public void setDiscoveryTTL(final String ttl) {
        Jonas4.logger.finest("[Discovery configuration] setting tlt : " + ttl);
        NotApplicableHelper.notApplicable("Discovery.TTL");
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Service mail configuration */
    /**
     * {@inheritDoc}
     */
    public void setMailFactoryType(final String factoryType) {
        Jonas4.logger.finest("[Mail Service Configuration] setting mail factory type : " + factoryType);
        this.mail.setType(factoryType);
        this.isMailActivated = true;
    }

    /**
     * {@inheritDoc}
     */
    public void setMailFactoryName(final String factoryName) {
        Jonas4.logger.finest("[Mail Service Configuration] setting mail factory name : " + factoryName);
        this.mail.setName(factoryName);
        this.isMailActivated = true;
    }

    /**
     * {@inheritDoc}
     */
    public void addMailMimme(final String name, final String host, final String mailto, final String subject) {
        Jonas4.logger.finest("[Mail Service Configuration] addition of a new mail mimme configuration file.");
        NotApplicableHelper.notApplicable("Mail.addMailMimme");
    }

    /**
     * {@inheritDoc}
     */
    public void addMailSession(final String name) {
        Jonas4.logger.finest("[Mail Service Configuration] addition of a new mail session configuration file.");
        NotApplicableHelper.notApplicable("Mail.addMailSession");
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Service WS configuration */
    /**
     * {@inheritDoc}
     */
    public void setWsdlPublisherFileName(final String fileName) {
        Jonas4.logger.finest("[WS Service configuration] setting wsdlPublisherFile name : " + fileName);
        this.wsdlPublisherFile.setName(fileName);
        this.isWsdlPublishActivated = true;
    }

    /**
     * {@inheritDoc}
     */
    public void setWsdlPublisherFileDirectory(final String fileDir) {
        Jonas4.logger.finest("[WS Service configuration] setting wsdlPublisherFile directory : " + fileDir);
        this.wsdlPublisherFile.setDir(fileDir);
        this.isWsdlPublishActivated = true;
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Service HA configuration */
    /**
     * {@inheritDoc}
     */
    public void setHaActivated(final Boolean activated) {
        Jonas4.logger.finest("[HA Service configuration] setting ha activated : " + activated);
        this.isHaActivated = activated;
    }

    /**
     * {@inheritDoc}
     */
    public void setHaMulticastAddress(final String multicastAddr) {
        Jonas4.logger.finest("[HA Service configuration] setting multicast address : " + multicastAddr);
        this.ha.setMcastAddr(multicastAddr);
    }

    /**
     * {@inheritDoc}
     */
    public void setHaMulticastPort(final String multicastPort) {
        Jonas4.logger.finest("[HA Service configuration] setting multicast port : " + multicastPort);
        this.ha.setMcastPort(multicastPort);
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Security manager configuration */
    /**
     * {@inheritDoc}
     */
    public void setSecurityManagerActivated(final Boolean activation) {
        Jonas4.logger.finest("[Security Manager configuration] setting isActivated : " + activation);
        if (this.jonasProperties != null) {
            this.jonasProperties.setSecurityManager(activation);
        } else {
            this.addReplacement("jonas.properties", "jonas.security.manager    true", "jonas.security.manager    "
                + activation.toString());
        }
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Ejb clustering configuration */
    /**
     * {@inheritDoc}
     */
    public void setEjbClusteringActivated(final Boolean activated) {
        Jonas4.logger.finest("[Ejb Clustering configuration] setting ejb clustering activated activated : " + activated);
        this.isEjbClusteringActivated = activated;
    }

    /**
     * {@inheritDoc}
     */
    public void setEjbClusteringMulticastAddress(final String multicastAddr) {
        Jonas4.logger.finest("[Ejb Clustering configuration] setting multicast address : " + multicastAddr);
        this.saveCmiMCastAddr = multicastAddr;
    }

    /**
     * {@inheritDoc}
     */
    public void setEjbClusteringMulticastPort(final String multicastPort) {
        Jonas4.logger.finest("[Ejb Clustering configuration] setting multicast port : " + multicastPort);
        this.saveCmiMCastPort = multicastPort;
    }

    /**
     * {@inheritDoc}
     */
    public void setEjbClusteringReplicationEnable(final Boolean enable) {
        Jonas4.logger.finest("[EJB Clustering configuration] : " + enable);
        NotApplicableHelper.notApplicable("Cmi.isReplicationEnable");
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Jms configuration */
    /**
     * {@inheritDoc}
     */
    public void setJmsPort(final String port) {
        Jonas4.logger.finest("[Jms configuration] setting port : " + port);
        this.jms.setPort(port);
    }

    /**
     * {@inheritDoc}
     */
    public void setJmsQueues(final String queue) {
        Jonas4.logger.finest("[Jms configuration] setting queue : " + queue);
        this.jms.setInitialQueues(queue);
    }

    /**
     * {@inheritDoc}
     */
    public void setJmsTopics(final String topic) {
        Jonas4.logger.finest("[Jms configuration] setting topic : " + topic);
        this.jms.setInitialTopics(topic);
    }

    /**
     * {@inheritDoc}
     */
    public void setJmsHost(final String host) {
        Jonas4.logger.finest("[Jms configuration] setting host : " + host);
        NotApplicableHelper.notApplicable("Jms.host");
    }

    /* ------------------------------- */

    /* ------------------------------- */
    /* Jdbc configuration */
    /**
     * {@inheritDoc}
     */
    public void addJdbcRA(final String raName, final JDBCConfiguration configuration) {
        JdbcRa jdbcRa = new JdbcRa();

        if (configuration.driverName != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting driverName: " + configuration.driverName);
            jdbcRa.setDriverName(configuration.driverName);
        }

        if (configuration.datasourceClass != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting datasourceClass: " + configuration.datasourceClass);
            NotApplicableHelper.notApplicable("JDBC.datasourceClass");
        }

        Jonas4.logger.finest("[Jdbc configuration] setting raName: " + raName);
        jdbcRa.setName(raName);

        if (configuration.checkLevel != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting checkLevel: " + configuration.checkLevel);
            NotApplicableHelper.notApplicable("JDBC.checkLevel");
        }

        if (configuration.testStatement != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting testStatement: " + configuration.testStatement);
            NotApplicableHelper.notApplicable("JDBC.testStatement");
        }

        if (configuration.jndiName != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting jndiName: " + configuration.jndiName);
            jdbcRa.setJndiName(configuration.jndiName);
        }

        if (configuration.mappername != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting mappername: " + configuration.mappername);
            jdbcRa.setMapperName(configuration.mappername);
        }

        if (configuration.password != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting password: " + configuration.password);
            jdbcRa.setPassword(configuration.password);
        }

        if (configuration.rarLink != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting rarLink: " + configuration.rarLink);
            NotApplicableHelper.notApplicable("JDBC.rarLink");
        }

        if (configuration.url != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting url: " + configuration.url);
            jdbcRa.setUrl(configuration.url);
        }

        if (configuration.user != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting user: " + configuration.user);
            jdbcRa.setUser(configuration.user);
        }

        if (configuration.poolInitialSize != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolInitialSize: " + configuration.poolInitialSize);
            NotApplicableHelper.notApplicable("JDBC.poolInitialSize");
        }

        if (configuration.poolMaximumSize != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolMaximumSize: " + configuration.poolMaximumSize);
            jdbcRa.setMaxPoolSize(configuration.poolMaximumSize);
        }

        if (configuration.poolMaximumAgeMinutes != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolMaximumAgeMinutes: " + configuration.poolMaximumAgeMinutes);
            try {
                jdbcRa.setConnMaxAge(configuration.poolMaximumAgeMinutes);
            } catch (NoSuchMethodError e) {
                NotApplicableHelper.notApplicable("JDBC.poolMaximumAgeMinutes");
            }
        }

        if (configuration.poolMaximumOpenTime != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolMaximumOpenTime: " + configuration.poolMaximumOpenTime);
            try {
                jdbcRa.setMaxOpenTime(configuration.poolMaximumOpenTime);
            } catch (NoSuchMethodError e) {
                NotApplicableHelper.notApplicable("poolMaximumOpenTime");
            }
        }

        if (configuration.poolMaximumNumberOfWaiters != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolMaximumNumberOfWaiters: "
                + configuration.poolMaximumNumberOfWaiters);
            try {
                jdbcRa.setMaxWaiters(configuration.poolMaximumNumberOfWaiters);
            } catch (NoSuchMethodError e) {
                NotApplicableHelper.notApplicable("JDBC.poolMaximumNumberOfWaiters");
            }
        }

        if (configuration.poolMaximumWaitTime != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolMaximumWaitTime: " + configuration.poolMaximumWaitTime);
            try {
                jdbcRa.setMaxWaitTime(configuration.poolMaximumWaitTime);
            } catch (NoSuchMethodError e) {
                NotApplicableHelper.notApplicable("JDBC.poolMaximumWaitTime");
            }
        }

        if (configuration.poolMinimumSize != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolMinimumSize: " + configuration.poolMinimumSize);
            NotApplicableHelper.notApplicable("JDBC.poolMinimumSize");
        }

        if (configuration.poolSamplingPeriod != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolSamplingPeriod: " + configuration.poolSamplingPeriod);
            NotApplicableHelper.notApplicable("JDBC.poolSamplingPeriod");
        }

        if (configuration.poolMaximumNumberOfPreparedStatements != null) {
            Jonas4.logger.finest("[Jdbc configuration] setting poolMaximumNumberOfPreparedStatements: "
                + configuration.poolMaximumNumberOfPreparedStatements);
            try {
                jdbcRa.setMaxPreparedStatementsSize(configuration.poolMaximumNumberOfPreparedStatements);
            } catch (NoSuchMethodError e) {
                NotApplicableHelper.notApplicable("JDBC.poolMaximumNumberOfPreparedStatements");
            }
        }

        this.jdbcRa.put(raName, jdbcRa);
    }

    /**
     * {@inheritDoc}
     */
    public void setSmartClientPort(final String port) {
        Jonas4.logger.finest("[Smartclient configuration] setting port: " + port);
        NotApplicableHelper.notApplicable("Smartclient.port");
    }

    /* ------------------------------- */

    /**
     * {@inheritDoc}
     */
    public void execute() {
        if (!this.isJonasRootSet) {
            throw new IllegalArgumentException("Jonas root is not set.");
        }
        if (!this.isJonasBaseSet) {
            throw new IllegalArgumentException("Jonas base is not set.");
        }

        if (this.isHttpActivated) {
            this.tomcat.addConfiguredHttp(this.http);
        }
        if (this.isHttpsActivated) {
            this.tomcat.addConfiguredHttps(this.https);
        }
        if (this.isAjpActivated) {
            this.tomcat.addConfiguredAjp(this.ajp);
        }
        if (this.isHttpReplicationActivated) {
            this.tomcat.addConfiguredCluster(this.cluster);
        }

        this.webContainer.addConfiguredTomcat(this.tomcat);

        if (this.isEjbClusteringActivated) {
            this.carol.setCmiMcastAddr(this.saveCmiMCastAddr);
            this.carol.setCmiMcastPort(this.saveCmiMCastPort);
        }
        if (!this.isProtocolsListSet) {
            this.carol.setProtocols("jrmp");
        }
        this.jonasBase.addConfiguredCarol(this.carol);
        if (this.isDbActivated) {
            this.jonasBase.addConfiguredDb(this.db);
        }
        if (this.isDiscoveryMaster) {
            this.discovery.setSourcePort("-1");
        }
        this.jonasBase.addConfiguredDiscovery(this.discovery);
        for (JdbcRa jdbcRa : this.jdbcRa.values()) {
            this.jonasBase.addConfiguredJdbcRa(jdbcRa);
        }
        this.jonasBase.addConfiguredJms(this.jms);
        if (this.jonasProperties != null) {
            this.jonasBase.addConfiguredJonasProperties(this.jonasProperties);
        } else {
            this.jonasBase.addConfiguredServices(this.services);
        }
        if (this.isMailActivated) {
            this.jonasBase.addConfiguredMail(this.mail);
        }
        this.jonasBase.addConfiguredWebContainer(this.webContainer);
        if (this.isWsdlPublishActivated) {
            this.wsdlPublish.addConfiguredFile(this.wsdlPublisherFile);
            this.jonasBase.addConfiguredWsdlPublish(this.wsdlPublish);
        }
        if (this.isHaActivated) {
            this.jonasBase.addTasks(this.ha);
        }

        this.jonasBase.execute();
        this.doReplacements(new File(this.jonasFileBase, "conf"));
    }

    /* --------------------------------- */
    /* Cluster daemon configuration part */

    /**
     * {@inheritDoc}
     */
    public void setCDClusterName(final String clusterName) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting cluster name : " + clusterName);
        this.clusterDaemon.setName(clusterName);
        this.clusterDaemon.updateName();
    }

    /**
     * {@inheritDoc}
     */
    public void setCDClusterDomain(final String clusterDomain) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting cluster domain name : " + clusterDomain);
        this.clusterDaemon.setDomainName(clusterDomain);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDProtocol(final String protocol) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting protocol : " + protocol);
        this.clusterDaemon.setProtocol(protocol);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDPort(final String port) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting port : " + port);
        this.clusterDaemon.setPort(port);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDDestDirPrefix(final String destDirPrefix) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting dest dir prefix : " + destDirPrefix);
        this.clusterDaemon.setDestDirPrefix(destDirPrefix);
        this.clusterDaemonTask.setDestDirPrefix(destDirPrefix);
        this.clusterDaemon.setCdDir(destDirPrefix);
        this.clusterDaemonTask.setCdDir(destDirPrefix);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDInteractionMode(final String interactionMode) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting interaction mode : " + interactionMode);
        this.clusterDaemon.setInteractionMode(interactionMode);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDJavaHome(final String javaHome) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting java home : " + javaHome);
        this.clusterDaemon.setJdk(javaHome);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDJonasRoot(final String jonasRoot) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting jonas root : " + jonasRoot);
        this.clusterDaemon.setJonasRoot(jonasRoot);
        this.clusterDaemonTask.setJonasRoot(new File(jonasRoot));
    }

    /**
     * {@inheritDoc}
     */
    public void setCDServerNamePrefix(final String serverNamePrefix) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting server name prefix : " + serverNamePrefix);
        this.clusterDaemon.setClusterNodesName(serverNamePrefix);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDXparam(final String xparam) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting xparam : " + xparam);
        this.clusterDaemon.setXprm(xparam);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDAutoBoot(final boolean autoBoot) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting auto boot : " + autoBoot);
        this.clusterDaemon.setAutoBoot(new Boolean(autoBoot).toString());
    }

    /**
     * {@inheritDoc}
     */
    public void setCDNbInstances(final int nbInsts) {
        Jonas4.logger.finest("[Cluster daemon configuration] setting nb instance : " + nbInsts);
        this.clusterDaemon.setInstNb(nbInsts);
        this.clusterDaemonTask.setWebInstNb(nbInsts);
        this.clusterDaemonTask.setEjbInstNb(0);
    }

    /**
     * {@inheritDoc}
     */
    public void setCDJOnASScript(String jonasScript) {
        Jonas4.logger.finest("[Cluster daemon configuration] jonas-script is not support");
        NotApplicableHelper.notApplicable("clusterdaemon.jonasScript");
    }

    /**
     * {@inheritDoc}
     */
    public void executeCDConf() {
        this.clusterDaemonTask.addConfiguredClusterDaemon(this.clusterDaemon);
        this.clusterDaemonTask.execute();
    }

    /**
     * {@inheritDoc}
     */
    public void setWrapperConfiguration(final String javaHome, final String wrapperHome, final String serviceName,
                                        final String serviceDescription, final Boolean verbosity, final String logfile,
                                        final String logfileRollmode) {
        Jonas4.logger.finest("Wrapper configuration is not support");
        NotApplicableHelper.notApplicable("wrapper.conf");
    }
    /* --------------------------------- */
}
